/* Real-Time Electrocardiogram Monitoring
   Common code
   config.c - Function for managing the project configuration file
   Calvin Walden
   2017-04-16
*/

#include <syslog.h>
#include <libconfig.h>

#include "config.h"
#include "rtecg.h"

struct rtecg_cfg rtecg_config;

/* Name of project-wide configuration file */
const char config_filename[] = "rtecg.cfg";



/* configLoad: Load and parse a configuration file using libconfig calls,
   put contents into struct rtecg_cfg */
int configLoad(void)
{
    int ret;
    config_t cfg;
    char filename_tmp[200];
    char msg_buffer[200];
    int tmp_integer;

    config_init(&cfg);

    /* Load the config file */
    /* Try /etc/rtecg/{filename} first */
    snprintf(filename_tmp, sizeof(filename_tmp),
             "/etc/rtecg/%s", config_filename);

    ret = config_read_file(&cfg, filename_tmp);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: %s: %d - %s",
                 filename_tmp, config_error_line(&cfg),
                 config_error_text(&cfg));
        syslog(LOG_NOTICE, msg_buffer);

        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: (Non-fatal) Unable to load %s, "
                 "trying something else",
                 filename_tmp);
        syslog(LOG_NOTICE, msg_buffer);

        /* If that doesn't work, try ./{filename} */
        snprintf(filename_tmp, sizeof(filename_tmp),
                 "./%s", config_filename);

        ret = config_read_file(&cfg, filename_tmp);
        if (!ret) {
            snprintf(msg_buffer, sizeof(msg_buffer),
                     "configLoad: %s: %d - %s",
                     filename_tmp, config_error_line(&cfg),
                     config_error_text(&cfg));
            syslog(LOG_ERR, msg_buffer);

            snprintf(msg_buffer, sizeof(msg_buffer),
                     "configLoad Unable to load %s, giving up.",
                     filename_tmp);
            syslog(LOG_NOTICE, msg_buffer);

            return 1;
        } else {
            snprintf(msg_buffer, sizeof(msg_buffer),
                     "configLoad: Loaded configuration data from %s",
                     filename_tmp);
            syslog(LOG_NOTICE, msg_buffer);
        }

    } else {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Loaded configuration data from %s",
                 filename_tmp);
        syslog(LOG_NOTICE, msg_buffer);    
    }

    /* Copy values/pointers into the rtecg_cfg struct */

    /* device name */
    ret = config_lookup_string(&cfg, "device_name", &rtecg_config.device_name);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve device_name from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* device id */
    ret = config_lookup_int(&cfg, "device_id", &tmp_integer);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve device_id from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }
    rtecg_config.device_id = tmp_integer;

    /* patient name */
    ret = config_lookup_string(&cfg, "patient_name", &rtecg_config.patient_name);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve patient_name from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* patient id */
    ret = config_lookup_int(&cfg, "patient_id", &tmp_integer);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve patient_id from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }
    rtecg_config.patient_id = tmp_integer;

    /* patient name */
    ret = config_lookup_string(&cfg, "patient_email", &rtecg_config.patient_email);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve patient_email from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* physician email */
    ret = config_lookup_string(&cfg, "physician_email", &rtecg_config.physician_email);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve physician_email from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* rtecg directory */
    ret = config_lookup_string(&cfg, "rtecg_dir", &rtecg_config.rtecg_dir);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve rtecg_dir from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }


    /* logfile directory */
    ret = config_lookup_string(&cfg, "logfile_dir", &rtecg_config.logfile_dir);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve logfile_dir from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* plot directory */
    ret = config_lookup_string(&cfg, "plot_dir", &rtecg_config.plot_dir);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve plot_dir from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* mail bin */
    ret = config_lookup_string(&cfg, "mail_bin", &rtecg_config.mail_bin);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve mail_bin from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* gnuplot bin */
    ret = config_lookup_string(&cfg, "gnuplot_bin", &rtecg_config.gnuplot_bin);
    if (!ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "configLoad: Unable to retrieve gnuplot_bin from %s",
                 filename_tmp);
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }
    
    return 0;
}

