/* Real-Time Electrocardiogram Monitoring
   Acquisition Daemon
   acquire.c - Functions for buffering and managing ECG data acquired over 
   serial port.
   Calvin Walden
   2017-03-20
*/

#include <syslog.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>

#include "acquire.h"
#include "file.h"
#include "serial.h"
#include "../../common/rtecg.h"

int16_t buffer_raw[BUFFER_SIZE] = {0};
uint16_t index_raw = 0;

int16_t buffer_filt[BUFFER_SIZE] = {0};
uint16_t index_filt = 0;

/* QRS peak index buffer */
uint16_t buffer_qrs[BUFFER_SIZE] = {0};
uint16_t index_qrs = 0;

/* PVC peak index buffer */
uint16_t buffer_pvc[BUFFER_SIZE] = {0};
uint16_t index_pvc = 0;



/* acquireLoop: Main loop of acquisition daemon. It polls the serial port for
   single bytes, checks if they are significant flags, and delegates further 
   data acquisition to other functions. 
   Input: None
   Output: None (directly, at least)
   Return: Nothing meaningful
*/
int acquireLoop(void) {
    uint16_t read_value;

    int counter_flag = 0;

    syslog(LOG_NOTICE, "acquireLoop: Data acquisition started.");

    /* Kinda-sorta state machine that first checks for flags to determine where 
       and how recieved data should be saved. */
    while (1) {
        read_value = serialRead8();

        switch (read_value) {
        case FLAG_START:    
            read_value = serialRead8();
            if (read_value == FLAG_CONST) {
                counter_flag++;
                if (counter_flag == 3) {
#ifdef DEBUG_ACQUIRE_LOOP
                    syslog(LOG_NOTICE, "acquireLoop: Start flag recieved");
#endif
                    counter_flag = 0;
                }
            } else {
                counter_flag = 0;
            }
            break;
        case FLAG_ARRHYTHMIA_DETECTED:
            /* Arrythmia was detected; need to immediately dump data and trigger
               communication daemon to send it. */
            read_value = serialRead8();
            if (read_value == FLAG_CONST) {
                counter_flag++;
                if (counter_flag == 3) {
#ifdef DEBUG_ACQUIRE_LOOP
                    syslog(LOG_NOTICE, "acquireLoop: Arrhythmia flag recieved");
#endif
                    counter_flag = 0;
                    /* Dump data + PVC peaks to be overlaid */
                    acquireDump(buffer_filt, &index_filt,
                                buffer_pvc, &index_pvc,
                                &logspec_filt);
                    /* WARNING: Until logic to determine the desired data 
                       source is implemented, we're just going to assume that
                       we're interested in dumping the filtered data. */
                    acquireHandleArrhythmia();
                }
            }
        case FLAG_SEND_FILT:
            read_value = serialRead8();
            if (read_value == FLAG_CONST) {
                counter_flag++;
                if (counter_flag == 3) {
#ifdef DEBUG_ACQUIRE_LOOP
                    syslog(LOG_NOTICE, "acquireLoop: Filtered data flag recieved");
#endif
                    counter_flag = 0;
                    acquireData(buffer_filt, &index_filt);
                }
            } else {
                counter_flag = 0;
            }
            break;
        case FLAG_SEND_RAW:
            read_value = serialRead8();
            if (read_value == FLAG_CONST) {
                counter_flag++;
                if (counter_flag == 3) {
#ifdef DEBUG_ACQUIRE_LOOP
                    syslog(LOG_NOTICE, "acquireLoop: Raw data flag recieved");
#endif
                    counter_flag = 0;
                    acquireData(buffer_raw, &index_raw);
                }
            } else {
                counter_flag = 0;
            }
            break;
        case FLAG_SEND_QRS:
            read_value = serialRead8();
            if (read_value == FLAG_CONST) {
                counter_flag++;
                if (counter_flag == 3) {
#ifdef DEBUG_ACQUIRE_LOOP
                    syslog(LOG_NOTICE, "acquireLoop: QRS peak flag recieved");
#endif
                    counter_flag = 0;
#ifdef TESTING_BUFFER_INDICES
                    acquireIndexData(buffer_qrs, &index_qrs, index_filt);
                    /* Assuming we want filtered data, using this index */
#endif
                }
            } else {
                counter_flag = 0;
            }
            break;
        case FLAG_SEND_PVC:
            read_value = serialRead8();
            if (read_value == FLAG_CONST) {
                counter_flag++;
                if (counter_flag == 3) {
#ifdef DEBUG_ACQUIRE_LOOP
                    syslog(LOG_NOTICE, "acquireLoop: PVC peak flag recieved");
#endif
                    counter_flag = 0;
#ifdef TESTING_BUFFER_INDICES
                    acquireIndexData(buffer_pvc, &index_pvc, index_filt);
                    /* Assuming we want filtered data, using this index */
#endif
                }
            } else {
                    counter_flag = 0;
            }
            break;
        case FLAG_STOP:
            read_value = serialRead8();
            if (read_value == FLAG_CONST) {
                counter_flag++;
                if (counter_flag == 3) {
#ifdef DEBUG_ACQUIRE_LOOP
                    syslog(LOG_NOTICE, "acquireLoop: Stop flag recieved");
#endif
                    counter_flag = 0;
                    /* Dump data + QRS peaks overlaid */
                    acquireDump(buffer_filt, &index_filt,
                                buffer_qrs, &index_qrs, 
                                &logspec_filt);
                }
            } else {
                counter_flag = 0;
            }
            break;
        case FLAG_CONST:
            break;
        default:
            break;
        }
    }

    return 0;
}



/* acquireData: Having recieved the appropriate flag in the main loop, 
   read the ECG data from the DSP into a given buffer, starting at a 
   specified index. 
   Input: buffer - pointer to buffer
          index - offset in buffer to start adding new data
   Output: buffer - will contain newly acquired data
           index - updated to last position of new data
   Return: *index, for convenience
*/
uint16_t acquireData(int16_t* buffer, uint16_t* index)
{
    uint16_t i;
    int16_t read_value;
    int counter_flag = 0;
#ifdef DEBUG_ACQUIRE
    char msg_buffer[200];
#endif

    /* When this function is called, the DSP is already transmitting its 
       buffer, so we read data into a local buffer here until we reach the 
       maximum buffer dump size or the FLAG_END_BUFFER flag is recieved. */

    /* It's reasonably safe to assume that the transmission will be 
       synchronized and the bytes will be recieved in the correct order, since
       that was established in acquireLoop() when the start flag was 
       detected. */

    for (i=0; i<SENSOR_BUFFER; i++) {
        read_value = serialRead16();

        buffer[*index + i] = read_value;

#ifdef DEBUG_ACQUIRE
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "buffer[%i + %i] = %i",
                 *index, i, buffer[*index+i]);
        syslog(LOG_NOTICE, msg_buffer);
#endif
    }

    while (1) {
        read_value = serialRead16();
#ifdef DEBUG_ACQUIRE
        snprintf(msg_buffer, sizeof(msg_buffer), "acquireData: read 0x%04x",
                 read_value);
        syslog(LOG_NOTICE, msg_buffer);
#endif
        if ((uint16_t)read_value == ((FLAG_END_BUFFER << 8) | FLAG_CONST)) {
            counter_flag++;
            if (counter_flag == 3) {
                syslog(LOG_NOTICE, "acquireData: Acquired complete buffer.");
                *index += i;
                break;
            }
        } else {
            syslog(LOG_ERR, "acquireData: Did not recieve entire "
                   "END_BUFFER flag.");
            break;
        }
    }

    return *index;
}


/* acquireIndexData(): Having recieved the appropriate flag from the main loop,
   recieve a list of indicies from the DSP, either for detected QRS or 
   PVC peaks.
   Input: buffer - buffer to contain indices
          index - offset in buffer to start adding new indicies
          base_index - The value added to recieved indicies, making them 
          relative to the most recently acquired data buffer. If this 
          isn't clear, studying the code may help (or make it worse.)
   Output: buffer - new values are added at offset "index"
           index - updated to point to new end of buffer, after new values 
           are added.
   Return value: *index, for convenience

   This functionality was never completed due to the implementation of this on 
   the DSP side, and doesn't work.
*/
#ifdef TESTING_BUFFER_INDICES
uint16_t acquireIndexData(uint16_t* buffer, uint16_t* index, uint16_t base_index)
{
    uint16_t i;
    int16_t read_value;
    int counter_flag = 0;
    char msg_buffer[200];

    /* This time, we read until we get an END_OF_BUFFER, since the number of
       peak indices is unknown. */
    i = 0;
    while (1) {
        read_value = serialRead16();
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "acquireIndexData: read_value = %04x", read_value);
        syslog(LOG_NOTICE, msg_buffer);

        if ((uint16_t)read_value == ((FLAG_END_BUFFER << 8) | FLAG_CONST)) {
            counter_flag++;
            if (counter_flag == 3) {
                syslog(LOG_NOTICE, "acquireIndexData: Acquired complete buffer");
                *index += i;
                break;
            }
        } else {
            if (counter_flag > 0) {
                /* If we're partially into an END sequence, we have a problem here */
                syslog(LOG_ERR, "acquireIndexData: Did not recieve entire "
                       "END_BUFFER flag");
                break;
            } else {
#if 0
                /* Otherwise, this is normal reading from the serial port, 
                   into the buffer. */
                
                /* We're reading index values, so we have to add the offset from 
                   previously dumped buffers because the values sent from the DSP are 
                   relative to the current dump, not our buffer here. */
                buffer[*index + i] = (read_value + base_index);
                snprintf(msg_buffer, sizeof(msg_buffer),
                         "index_buffer[%i + %i] = %i",
                         *index, i, buffer[*index+i]);
                syslog(LOG_NOTICE, msg_buffer);

                i++;
#endif
            }
        }
    }

    return *index;
}
#endif



/* acquireDump: If the specified buffer is sufficiently full, dump its 
   contents to a logfile and reset its index variable. Otherwise, leave
   *index untouched.
   Input: buffer - buffer to check and dump data from
          index - current index in the buffer. Condition for dumping is checked
                  based on that. This becomes the first column of data in the 
                  log file.
          peak_buffer - supplementary buffer of peak index data, filled by 
                        acquireIndexData(). This becomes the second column of 
                        data in the log file.
          peak_len - length of peak_buffer
          log - logfile specification to which to dump data
                (struct defined in file.h)
   Output: None directly
   Return: *index, which may or may not have been changed.
*/
uint16_t acquireDump(int16_t* buffer, uint16_t* index,
                     uint16_t* peak_buffer, uint16_t* peak_len,
                     struct logspec* log)
{
    int ret;
#ifdef DEBUG_ACQUIRE
    char msg_buffer[100];
#endif

#ifdef DEBUG_ACQUIRE
    snprintf(msg_buffer, sizeof(msg_buffer),
             "acquireDump: dumping buffer (index = %x of %x)", *index, BUFFER_SIZE);
    syslog(LOG_NOTICE, msg_buffer);
#endif

    if (*index == BUFFER_SIZE) {
        ret = fileWriteLog(buffer, peak_buffer, *peak_len, log);
        if (ret) {
            syslog(LOG_ERR, "acquireDump: Error at fileWriteLog().");
            return *index; /* This may or may not be the most appropriate thing
                              to do here. */
        }

        syslog(LOG_NOTICE, "acquireDump: Dumped buffer to file");

        *peak_len = 0;
        *index = 0;
    }
    
    return *index;
}



/* acquireHandleArrthymia(): Called by main loop if a flag indicating that the 
   DSP detected an arrhythmia is recieved. This function gets the PID of the 
   communication daemon (assuming that it's running), and sends it SIGUSR2 to 
   trigger sending an urgent ECG report message.
   Input: None
   Output: None
   Return: 0 on success, 1 on error encountered
*/
int acquireHandleArrhythmia(void)
{
    char msg_buffer[200];
    char pipe_buffer[10];
    pid_t comm_pid;
    FILE* fd;

    /* Get the PID of rtecg_comm by opening a pipe and running `pidof` */
    fd = popen("pidof rtecg_comm", "r");
    if (fd == NULL) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "acquireHandleArrhythmia: Error at popen(): %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    fgets(pipe_buffer, 10, fd);
    comm_pid = strtoul(pipe_buffer, NULL, 10);

    /* This way we'll know if rtecg_comm is even running. */
    if (comm_pid == 0) {
        syslog(LOG_ERR, "acquireHandleArrhytmia: CRITICAL ERROR: "
               "rtecg_comm is not running!");
        /* TODO: Should try and start rtecg_comm here and try again */
        return 1;
    }

    pclose(fd);

    snprintf(msg_buffer, sizeof(msg_buffer),
             "acquireHandleArrhythmia: Sending signal SIGUSR2 to rtecg_comm, pid %i",
             comm_pid);
    syslog(LOG_NOTICE, msg_buffer);

    /* Send the signal, SIGUSR2, to send an urgent email. */
    kill(comm_pid, SIGUSR2);

    return 0;
}
