/* Real-Time Electrocardiogram Monitoring
   Acquisition Daemon
   acquire.h - Functions for buffering and managing ECG data acquired over 
   serial port.
   Calvin Walden
   2017-03-20
*/

#ifndef ACQUIRE_H
#define ACQUIRE_H

#include <stdint.h>

#include "file.h"
#include "../../common/rtecg.h"

/* #define DEBUG_ACQUIRE_LOOP */
/* #define DEBUG_ACQUIRE */
/* #define TESTING_BUFFER_INDICES */
#define FILE_HISTORY 5 /* Minutes of data for each log file */



/* acquireLoop: Main loop of acquisition daemon. It polls the serial port for
   single bytes, checks if they are significant flags, and delegates further 
   data acquisition to other functions. 
   Input: None
   Output: None (directly, at least)
   Return: Nothing meaningful
*/
int acquireLoop(void);



/* acquireData: Having recieved the appropriate flag in the main loop, 
   read the ECG data from the DSP into a given buffer, starting at a 
   specified index. 
   Input: buffer - pointer to buffer
          index - offset in buffer to start adding new data
   Output: buffer - will contain newly acquired data
           index - updated to last position of new data
   Return: *index, for convenience
*/
uint16_t acquireData(int16_t* buffer, uint16_t* index);


/* acquireIndexData(): Having recieved the appropriate flag from the main loop,
   recieve a list of indicies from the DSP, either for detected QRS or 
   PVC peaks.
   Input: buffer - buffer to contain indices
          index - offset in buffer to start adding new indicies
          base_index - The value added to recieved indicies, making them 
          relative to the most recently acquired data buffer. If this 
          isn't clear, studying the code may help (or make it worse.)
   Output: buffer - new values are added at offset "index"
           index - updated to point to new end of buffer, after new values 
           are added.
   Return value: *index, for convenience

   This functionality was never completed due to the implementation of this on 
   the DSP side, and doesn't work.
*/
uint16_t acquireIndexData(uint16_t* buffer, uint16_t* index, uint16_t base_index);



/* acquireDump: If the specified buffer is sufficiently full, dump its 
   contents to a logfile and reset its index variable. Otherwise, leave
   *index untouched.
   Input: buffer - buffer to check and dump data from
          index - current index in the buffer. Condition for dumping is checked
                  based on that. This becomes the first column of data in the 
                  log file.
          peak_buffer - supplementary buffer of peak index data, filled by 
                        acquireIndexData(). This becomes the second column of 
                        data in the log file.
          peak_len - length of peak_buffer
          log - logfile specification to which to dump data
                (struct defined in file.h)
   Output: None directly
   Return: *index, which may or may not have been changed.
*/
uint16_t acquireDump(int16_t* buffer, uint16_t* index,
                     uint16_t* peak_buffer, uint16_t* peak_len,
                     struct logspec* log);



/* acquireHandleArrthymia(): Called by main loop if a flag indicating that the 
   DSP detected an arrhythmia is recieved. This function gets the PID of the 
   communication daemon (assuming that it's running), and sends it SIGUSR2 to 
   trigger sending an urgent ECG report message.
   Input: None
   Output: None
   Return: 0 on success, 1 on error encountered
*/
int acquireHandleArrhythmia(void);



#endif /* ACQUIRE_H */

