/* AVR I2C Data acquisition simulator
   Real-time Electrocardiogram Monitoring
   real-time-ecg/raspberry_pi/acquisition_daemon/avr-testing/main.c
   Calvin Walden
   2017-01-30

   This program simulates interaction between the Raspberry Pi and the 
   Texas Instruments C5515 on the I2C bus by acting as a slave device. 
   It was written to run on the Atmel ATmega8 family of microcontrollers.

   Remember that for each twiTransmitByte(), the I2C master is expected to
   perform a read operation.

   This program makes use of modified code originally from 
   chips_I2C_Slave_tutorial by Nels D. "Chip" Pearson, available at
   <https://github.com/CmdrZin/chips_I2C_Slave_tutorial>.
   It is distributed under the MIT License, as follows:

   * The MIT License (MIT)
   *
   * Copyright (c) 2016 Nels D. "Chip" Pearson (aka CmdrZin)
   *
   * Permission is hereby granted, free of charge, to any person obtaining a copy
   * of this software and associated documentation files (the "Software"), to deal
   * in the Software without restriction, including without limitation the rights
   * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   * copies of the Software, and to permit persons to whom the Software is
   * furnished to do so, subject to the following conditions:
   *
   * The above copyright notice and this permission notice shall be included in all
   * copies or substantial portions of the Software.

   */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <stdlib.h>

#include "twiSlave.h"
#include "../../../common/protocol.h"
#include "raw_data.h"
#include "helper.h"

int main(void)
{
    uint8_t i;
    uint8_t rxdata;
    uint8_t header_size;
    struct dsp_header header = { 0 };

    struct qrs_complex qrs;

    static uint16_t* payload_data;
    static uint16_t payload_checksum = 0;
    uint8_t random_hi;
    uint8_t random_lo;

    int ack;

    /* Disable watchdog timer */
    wdt_disable();

    /* Initialize I2C stuff */
    twiSlaveInit(DSP_SIM_ADDRESS);
    sei();
    twiSlaveEnable();

    /* Make up some data */
    header.num_qrs = 1;
    header.qrs = &qrs;
    header.result = rand();
    header.eot = EOT;

    qrs.q = 334;
    qrs.r = 350;
    qrs.s = 359;
        
    header.checksum ^= qrs.q;
    header.checksum ^= qrs.r;
    header.checksum ^= qrs.s;
    header.checksum ^= header.num_qrs;
    header.checksum ^= header.result;
    header.checksum ^= header.eot;

    for (i=0; i<BUFFER_TESTING_SAMPLES; i++) {
	payload_checksum ^= raw_data[i];
    }

    flushTwiBuffers();

    while (1) {
        if (twiDataInReceiveBuffer()) {
            rxdata = twiReceiveByte();

            switch (rxdata) {
            case CMD_POLL:
		flushTwiBuffers();
                if (rand() % 2) {
                    /* Put the data together for this transaction */
                    header_size = sizeof(struct dsp_header) +
			sizeof(struct qrs_complex)*header.num_qrs;

                    twiTransmitByte(DSP_POLL_READY);
                    twiTransmitByte(header_size);
		    twiTransmitWord(BUFFER_TESTING_SIZE);
                } else {
                    twiTransmitByte(DSP_POLL_NOTREADY);
                    twiTransmitByte(0);
                    twiTransmitWord(0);
                }
                break;
            case CMD_ACQUIREHEADER:
		do {
		    flushTwiBuffers();
		    /* Transmit checksum */
		    twiTransmitWord(header.checksum);
		    /* Transmit number of QRS complexes */
		    twiTransmitByte(header.num_qrs);
		    /* Transmit QRS data */
		    twiTransmitWord(qrs.q);
		    twiTransmitWord(qrs.r);
		    twiTransmitWord(qrs.s);
		    /* Transmit result */
		    twiTransmitWord(header.result);
		    /* Transmit EOT */
		    twiTransmitByte(header.eot);

		    /* Check for acknowledgement */
		    while(!twiDataInReceiveBuffer()) {}
		    ack = twiReceiveByte();
		    if (ack == 0) {
			break;
		    } else if (ack == CMD_RESET) {
			reset();
			break;
		    }
		    
		} while (1);
                break;
            case CMD_ACQUIREPAYLOAD:
		do {
		    flushTwiBuffers();
		    /* Send data checksum */
		    twiTransmitWord(payload_checksum);
		    /* Send data */
		    for (i=0; i<BUFFER_TESTING_SAMPLES; i++) {
			twiTransmitWord(raw_data[i]);
		    }
		    /* Check for acknowledgement, resend if nack */
		    while(!twiDataInReceiveBuffer()) {}
		    ack = twiReceiveByte();
		    if (ack == 0) {
			break;
		    } else if (ack == CMD_RESET) {
			reset();
			break;
		    }
		} while (1);
                break;
	    case CMD_RESET:
		reset();
		break;
            default:
                twiTransmitByte(0);
		break;
            }
        }
    }

    return 0;
}




