/* AVR I2C Data acquisition simulator
   Real-time Electrocardiogram Monitoring
   real-time-ecg/raspberry_pi/acquisition_daemon/avr-testing/twiSlave.h
   Calvin Walden
   2017-01-29

   This program simulates interaction between the Raspberry Pi and the 
   Texas Instruments C5515 on the I2C bus by acting as a slave device. 
   It was written to run on the Atmel ATmega8 family of microcontrollers.

   This program makes use of modified code originally from 
   chips_I2C_Slave_tutorial by Nels D. "Chip" Pearson, available at
   <https://github.com/CmdrZin/chips_I2C_Slave_tutorial>.
   It is distributed under the MIT License, as follows:

   * The MIT License (MIT)
   *
   * Copyright (c) 2016 Nels D. "Chip" Pearson (aka CmdrZin)
   *
   * Permission is hereby granted, free of charge, to any person obtaining a copy
   * of this software and associated documentation files (the "Software"), to deal
   * in the Software without restriction, including without limitation the rights
   * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   * copies of the Software, and to permit persons to whom the Software is
   * furnished to do so, subject to the following conditions:
   *
   * The above copyright notice and this permission notice shall be included in all
   * copies or substantial portions of the Software.

   Substantial changes made:
   - Fixed errors when compiling using avr-gcc
   - Reformatted code and comments to match project style and follow the 
   80-column rule

*/

#ifndef TWI_SLAVE_H
#define TWI_SLAVE_H

#include <stdbool.h>

#define TWI_RX_BUFFER_SIZE (256)
#define TWI_RX_BUFFER_MASK (TWI_RX_BUFFER_SIZE - 1)

#define TWI_TX_BUFFER_SIZE  (256)
#define TWI_TX_BUFFER_MASK  (TWI_TX_BUFFER_SIZE - 1)


/* TWI Slave Receiver status codes */
/*  - Own SLA+W has been received ACK has been returned */  
#define TWI_SRX_ADR_ACK             0x60
/*  - Own SLA+W has been received; ACK has been returned */
#define TWI_SRX_ADR_ACK_M_ARB_LOST  0x68
/*  - General call address has been received; ACK has been returned */
#define TWI_SRX_GEN_ACK             0x70
/*  - General call address has been received; ACK has been returned */
#define TWI_SRX_GEN_ACK_M_ARB_LOST  0x78
/*  - Previously addressed with own SLA+W; data has been received; 
      ACK has been returned */
#define TWI_SRX_ADR_DATA_ACK        0x80
/*  - Previously addressed with own SLA+W; data has been received; 
      NOT ACK has been returned */
#define TWI_SRX_ADR_DATA_NACK       0x88
/*  - Previously addressed with general call; data has been received; 
      ACK has been returned */
#define TWI_SRX_GEN_DATA_ACK        0x90
/*  - Previously addressed with general call; data has been received; 
      NOT ACK has been returned */
#define TWI_SRX_GEN_DATA_NACK       0x98
/*  - A STOP condition or repeated START condition has been received 
      while still addressed as Slave */
#define TWI_SRX_STOP_RESTART        0xA0  


/* TWI Slave Transmitter status codes */
/*  - Own SLA+R has been received; ACK has been returned */
#define TWI_STX_ADR_ACK             0xA8
/*  - Own SLA+R has been received; ACK has been returned */
#define TWI_STX_ADR_ACK_M_ARB_LOST  0xB0
/*  - Data byte in TWDR has been transmitted; ACK has been received */
#define TWI_STX_DATA_ACK            0xB8
/*  - Data byte in TWDR has been transmitted; NOT ACK has been received */
#define TWI_STX_DATA_NACK           0xC0
/*  - Last byte in TWDR has been transmitted (TWEA = 0); 
      ACK has been received */
#define TWI_STX_DATA_ACK_LAST_BYTE  0xC8  

/* TWI Miscellaneous status codes */
/*  - No relevant state information available; TWINT = 0 */
#define TWI_NO_STATE                0xF8
/*  - Bus error due to an illegal START or STOP condition */
#define TWI_BUS_ERROR               0x00  


/* Private functions */
void flushTwiBuffers(void);

/* Public functions */
void twiSlaveInit(uint8_t adrs);
void twiSlaveEnable(void);

void twiTransmitByte(uint8_t data);
void twiTransmitWord(uint16_t data);
uint8_t twiReceiveByte(void);

bool twiDataInReceiveBuffer(void);
bool twiDataInTransmitBuffer(void);
void twiClearOutput(void);
void twiStuffRxBuf(uint8_t data);

#endif /* TWI_SLAVE_H */
