/* Real-Time Electrocardiogram Monitoring
   Acquisition Daemon
   file.c - ECG data log file writing and creation operations
   Calvin Walden
   2017-03-20
*/

#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "../../common/config.h"
#include "acquire.h"
#include "file.h"
#include "helper.h"

/* Filtered data logfile specification */
struct logspec logspec_filt;
char filt_basename[] = "data_filt";
char filt_name[] = "Filtered ECG Data";
    
/* Raw data logfile specification */
struct logspec logspec_raw;
char raw_basename[] = "data_raw";
char raw_name[] = "Raw ECG Data";



/* fileInitializeLogspecs(): Initialize the logspec structures with default 
   values defined above.
   Input: None
   Output: None
   Return: None
*/
int fileInitializeLogspecs(void)
{
    /* Filtered data logfile */
    logspec_filt.filename_current[0] = 0;
    strncpy(logspec_filt.filename_base, filt_basename, strlen(filt_basename));
    strncpy(logspec_filt.name, filt_name, strlen(filt_name));
    logspec_filt.last_modified = 0;

    /* Raw data logfile */
    logspec_raw.filename_current[0] = 0;
    strncpy(logspec_raw.filename_base, raw_basename, strlen(raw_basename));
    strncpy(logspec_raw.name, raw_name, strlen(raw_name));
    logspec_raw.last_modified = 0;

    return 0;
}



/* fileMakeFilename(): Using the current time, the last modified time of the 
   given logfile (logspec->last_modified), and the basename
   (logspec->filename_base), either generate a new filename if enough time has 
   elapsed (FILE_HISTORY in acquire.h), or keep the current one.
   Input: logfile (multiple elements)
   Output: logfile->filename_current
   Return: None
*/
int fileMakeFilename(struct logspec* logfile)
{
    time_t current_time;
    time_t prev_time;
    struct tm timeinfo;
    char filename_buffer[200];
    
    prev_time = logfile->last_modified;

    /* Get current time */
    current_time = time(NULL);
    
    /* If last_modified is >= 15 minutes from current time,
       update filename. */
    if ((current_time - prev_time) >= (60*FILE_HISTORY)) {
        timeinfo = *localtime(&current_time);

        snprintf(filename_buffer, sizeof(filename_buffer),
                 "%s%s_%04i%02i%02i_%02i%02i%02i.log",
                 rtecg_config.logfile_dir,
                 logfile->filename_base,
                 timeinfo.tm_year + 1900,
                 timeinfo.tm_mon + 1,
                 timeinfo.tm_mday,
                 timeinfo.tm_hour,
                 timeinfo.tm_min,
                 timeinfo.tm_sec);
        
        strncpy(logfile->filename_current, filename_buffer,
                strlen(filename_buffer));

        /* Update modification time */
        logfile->last_modified = current_time;
    }

    return 0;
}



/* fileWriteLog(): Write the contents of buffer to the logspec's current file. 
   This is called by acquireDump() when the buffer is determined to be full.
   Input: buffer - buffer to write to log file, first column
          peak_buffer - supplementary peak indicies, second column
          peak_len - length of peak_buffer
          logfile - specification for logfile, provides filename
*/
int fileWriteLog(int16_t* buffer,
                 uint16_t* peak_buffer, uint16_t peak_len,
                 struct logspec* logfile)
{
    int fd;
    int ret;
    int i;
    char msg_buffer[200];
    char write_buffer[200];

    fileMakeFilename(logfile);

    syslog(LOG_NOTICE, "fileWriteLog: Writing to file %s",
        logfile->filename_current);

    fd = open(logfile->filename_current, (O_APPEND | O_RDWR | O_CREAT), 0666);
    if (fd == -1) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "fileWriteLog (%s): Error at open(): %s",
                 logfile->filename_current,
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* Write data */
    for (i=0; i<BUFFER_SIZE; i++) {
        snprintf(write_buffer, sizeof(write_buffer),
                 "%05i   %i  \n", buffer[i],
                 helperTranslateIndexValue(i, peak_buffer, peak_len));
        ret = write(fd, write_buffer, strlen(write_buffer));
        if (ret == -1) {
            snprintf(msg_buffer, sizeof(msg_buffer),
                     "fileWriteLog (%s): Error at write(): %s",
                     logfile->filename_current,
                     strerror(errno));
            syslog(LOG_ERR, msg_buffer);
            close(fd);
            return 1;
        } else if (ret < strlen(write_buffer)) {
            snprintf(msg_buffer, sizeof(msg_buffer),
                     "fileWriteLog (%s): Error at write() (2).",
                     logfile->filename_current);
            syslog(LOG_ERR, msg_buffer);
            close(fd);
            return 1;
        }
    }

    close(fd);

    return 0;
}

