/* Real-Time Electrocardiogram Monitoring
   Acquisition Daemon
   serial.c - Serial communication functions
   Calvin Walden
   2017-03-20
*/

#include <errno.h>
#include <fcntl.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/ioctl.h>

#include "../../common/rtecg.h"
#include "serial.h"

static int fd_serial;



/* serialSetup(): Initialize the serial port, SERIAL_DEVICE in rtecg.h, to use
   SERIAL_BAUD with 8n1 protocol.
   Input: None
   Output: None
   Return: 0 on success, 1 on failure

   Major reference: http://stackoverflow.com/questions/6947413 
*/
int serialSetup(void) {
    int ret;
    char msg_buffer[200];

    /* Open the serial port device and get a file descriptor. */
    fd_serial = open(SERIAL_DEVICE, O_RDWR | O_NOCTTY | O_SYNC);
    if (fd_serial < 0) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "serialSetup: Error at open(%s): %s",
                 SERIAL_DEVICE, strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* Set interface attributes */
    struct termios tty;

    ret = tcgetattr(fd_serial, &tty);
    if (ret < 0) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "serialSetup: Error at tcgetattr(): %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    cfsetospeed(&tty, (speed_t)serialTranslateBaud(SERIAL_BAUD));
    cfsetispeed(&tty, (speed_t)serialTranslateBaud(SERIAL_BAUD));

    tty.c_cflag |= (CLOCAL | CREAD); /* Ignore modem controls */
    tty.c_cflag &= ~CSIZE;
    tty.c_cflag |= CS8;              /* 8 bits per character */
    tty.c_cflag &= ~PARENB;          /* No parity bit */
    tty.c_cflag &= ~CSTOPB;          /* One stop bit */
    tty.c_cflag &= ~CRTSCTS;         /* No hardware flow control */

    /* "Setup for non-canonical mode" - If this works, leave it. */
    tty.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
    tty.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
    tty.c_oflag &= ~OPOST;

    /* fetch bytes as they become available */
    tty.c_cc[VMIN] = 1;
    tty.c_cc[VTIME] = 1;

    ret = tcsetattr(fd_serial, TCSANOW, &tty);
    if (ret < 0) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "serialSetup: Error at tcsetattr(): %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    return 0;
}



/* serialClose(): Close the serial port. */
int serialClose(void)
{
    int ret;

    ret = close(fd_serial);
    if (ret) {
        syslog(LOG_ERR, "serialClose: Error at close().");
        return 1;
    }

    fd_serial = 0;
    
    return 0;
}



/* serialRead(): Read a string, of length len, of bytes into buffer
   Input: buffer, len
   Output: buffer
   Return: 0 on success (len bytes read), 1 on failure (less than len bytes 
   read)
*/
int serialRead(uint8_t* buffer, int len)
{
    int read_len;
    char msg_buffer[4096];
    
    read_len = read(fd_serial, buffer, len);
    if (read_len > 0) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "Read %d: \"%s\"",
                 read_len, buffer);
        syslog(LOG_NOTICE, msg_buffer);
    } else {
        return 1;
    }


    return 0;
}



/* serialRead16(): Read a single signed 16-bit word from the serial port. If 
   two bytes are not available in the system's buffer, wait for both of them. 
   This function will block the running thread if this happens.
   Expected byte order: High byte first, low byte second
   Input: None
   Output: None
   Return: Read value, or 0 on failure (accompanied by error message in syslog)
*/
int16_t serialRead16(void)
{
    uint8_t bytes[2];
    int16_t word;

    int read_len;
    int bytes_available;

#ifdef DEBUG_SERIAL
    char msg_buffer[100];
#endif


    while (1) {
        /* Check how many bytes are available to read */
        ioctl(fd_serial, FIONREAD, &bytes_available);
#ifdef DEBUG_SERIAL
        syslog(LOG_NOTICE, "serialRead16: Waiting for bytes...");
#endif

        if (bytes_available < 2) {
            /* Wait for two bytes */
            continue;
        } else {
            /* Otherwise, read both of them */
            read_len = read(fd_serial, bytes, 2);

            if (read_len == 2) {
                /* Success, break out of loop */
                break;
            } else {
                syslog(LOG_ERR, "serialRead16: Could not read 2 bytes.");
                return 0;
            }
        }
    }

    word = (bytes[0] << 8) | bytes[1];

#ifdef DEBUG_SERIAL
    snprintf(msg_buffer, sizeof(msg_buffer),
             "serialRead16: Read word 0x%04x",
             word);
    syslog(LOG_NOTICE, msg_buffer);
#endif

    return word;
}



/* serialRead8(): Read a single, unsigned byte from the serial port.
   Input: None
   Output: None
   Return: Read value, or 0 if a byte could not be read.
*/
uint8_t serialRead8(void)
{
    uint8_t buffer;
    int read_len;
#ifdef DEBUG_SERIAL
    char msg_buffer[100];
#endif

    read_len = read(fd_serial, &buffer, 1);
    if (read_len < 1) {
        return 0;
    }

#ifdef DEBUG_SERIAL
    snprintf(msg_buffer, sizeof(msg_buffer),
             "serialRead8: Read byte 0x%02x",
             buffer);
    syslog(LOG_NOTICE, msg_buffer);
#endif

    return buffer;
}



/* serialTranslateBaud(): Helper function that returns the "B----" constant 
   from <termbits.h> corresponding to integer_baud that cfsetospeed() wants. 
   Input: integer_baud
   Output: None
   Return: <termbits.h> constant, or -1 if undefined baud is specified
*/
int serialTranslateBaud(int integer_baud)
{
    char msg_buffer[50];

    switch(integer_baud) {
    case 0:
        return B0;
    case 50:
        return B50;
    case 75:
        return B75;
    case 110:
        return B110;
    case 134:
        return B134;
    case 150:
        return B150;
    case 200:
        return B200;
    case 300:
        return B300;
    case 600:
        return B600;
    case 1200:
        return B1200;
    case 1800:
        return B1800;
    case 2400:
        return B2400;
    case 4800:
        return B4800;
    case 9600:
        return B9600;
    case 19200:
        return B19200;
    case 38400:
        return B38400;
    case 57600:
        return B57600;
    case 115200:
        return B115200;
    case 230400:
        return B230400;
    case 460800:
        return B460800;
        /* Add more exotic speeds as you see fit. */
    default:
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "serialTranslateBaud: Unknown baud value %i",
                 integer_baud);
        syslog(LOG_ERR, msg_buffer);
        return -1;
    }

    return -1;
}

