/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   comm.c - Communication functions, including main loop
   Calvin Walden
   2017-03-20
*/

#define _XOPEN_SOURCE 700
#include <dirent.h>
#include <libgen.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include "../../common/config.h"
#include "comm.h"
#include "gnuplot.h"
#include "helper.h"
#include "mail.h"
#include "signalhandler.h"



/* commLoop(): Main loop for communication daemon. The loop polls for flags
   set by the signal handler function, all found in signalhandler.h 
   Input: None
   Output: None
*/
int commLoop(void)
{
    syslog(LOG_NOTICE, "commLoop: Waiting for signals.");

    /* Wait for signals to be received */
    while(1) {
        sleep(1);

        if (sigusr1_flag == 1) {
#ifdef DEBUG_COMM
            syslog(LOG_NOTICE, "signalHandler: Recieved SIGUSR1, "
                   "calling commTransmit().");
#endif
            commTransmit();
            sigusr1_flag = 0;
        }

        if (sigusr2_flag == 1) {
#ifdef DEBUG_COMM
            syslog(LOG_NOTICE, "signalHandler: Recieved SIGUSR2, "
                   "calling commTransmitUrgent().");
#endif
            commTransmitUrgent();
            sigusr2_flag = 0;
        }
    }

    return 0;
}



/* commTransmit(): Transmit last 15 minutes of data, probably routine and 
   called by cron(8) 
   Output: None
   Input: None
   Return: 0 on success, 1 on failure
*/
int commTransmit(void)
{
    int ret;
    char logfile[PROJECT_PATH_MAX] = {0};
    char logfile_copy[PROJECT_PATH_MAX] = {0};
    char msg_buffer[PROJECT_PATH_MAX];
    char pngfile[PROJECT_PATH_MAX] = {0};

    char date_buffer[30];

    char message_subject[1024];
    char message_body[1024];

    /* Chose which ECG data file should be sent. This will be the most recently
       created file, or the last file if sorted by name alphabetically. */
    ret = commGetNewestLogfile(logfile);
    if (ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "commTransmit: Error at commGetFilename().");
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* Generate a filename for PNG file that will be the ECG data plot */
    /* Start by making copy of the logfile path that we can mutiliate 
       into the final file path */
    strncpy(logfile_copy, logfile, strlen(logfile));
    /* Cut the file path string at the extension ".log" with a NULL 
       character */
    *(strstr(logfile_copy, ".log")) = 0;
    /* Then concatenate the path and basename, replacing .log with .png */    
    snprintf(pngfile, PROJECT_PATH_MAX, "%s%s", rtecg_config.plot_dir,
             strcat(basename(logfile_copy), ".png"));

    /* Generate a plot */
    ret = gnuplotPlotFile(logfile, pngfile);
    if (ret) {
        syslog(LOG_ERR, "commTransmit: (Non-fatal) Error at gnuplotPlotFile()."
               " Email will not contain ECG data plot.");
    }

    /* Build subject */
    snprintf(message_subject, sizeof(message_subject),
             "ECG Monitor Report %s (%03i:%i)",
             helperGetISODateTime(date_buffer),
             rtecg_config.device_id, rtecg_config.patient_id);

    /* Build body */
    snprintf(message_body, sizeof(message_body),
             "Routine ECG monitor report for %s (%i), generated %s by %s.\n\n"
             "Attached are 15 minutes of raw data and a plot of the last 30 "
             "seconds of recorded ECG activity.",
             rtecg_config.patient_name,
             rtecg_config.patient_id,
             helperGetISODateTime(date_buffer),
             rtecg_config.device_name);

    /* Send message with attachments */
    ret = sendMailAttachments((char*)rtecg_config.physician_email,
                              message_subject, message_body,
                              2, pngfile, logfile);
    if (ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "commTransmit(): Error at sendMailAttachments().");
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    snprintf(msg_buffer, sizeof(msg_buffer),
             "commTransmit: Sent message with %s and %s attached.",
             logfile, pngfile);
    syslog(LOG_NOTICE, msg_buffer);

    return 0;
}



/* commTransmitUrgent(): Urgent data transmission, most likely following an 
   arrhythmia being detected by the DSP and logged by rtecg_acquisition(1).
   This is roughly the same as commTransmit() with some different strings,
   but its separation allows for more functionality in the event of an
   emergency.
   Input: None
   Output: None
   Return: 0 on success, 1 on failure
*/
int commTransmitUrgent(void)
{
    int ret;
    char logfile[PROJECT_PATH_MAX] = {0};
    char logfile_copy[PROJECT_PATH_MAX] = {0};
    char msg_buffer[PROJECT_PATH_MAX];
    char pngfile[PROJECT_PATH_MAX] = {0};

    char date_buffer[30];

    char message_subject[1024];
    char message_body[1024];

    /* Chose which ECG data file should be sent. This will be the most recently
       created file, or the last file if sorted by name alphabetically. */
    ret = commGetNewestLogfile(logfile);
    if (ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "commTransmitUrgent: Error at commGetFilename().");
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* Generate a filename for PNG file that will be the ECG data plot */
    /* Start by making copy of the logfile path that we can mutiliate 
       into the final file path */
    strncpy(logfile_copy, logfile, strlen(logfile));
    /* Cut the file path string at the extension ".log" with a NULL 
       character */
    *(strstr(logfile_copy, ".log")) = 0;
    /* Then concatenate the path and basename, replacing .log with .png */    
    snprintf(pngfile, PROJECT_PATH_MAX, "%s%s", rtecg_config.plot_dir,
             strcat(basename(logfile_copy), ".png"));

    /* Generate a plot */
    ret = gnuplotPlotFile(logfile, pngfile);
    if (ret) {
        syslog(LOG_ERR, "commTransmitUrgent: (Non-fatal) Error at gnuplotPlotFile()."
               " Email will not contain PNG file.");
    }

    /* Build subject */
    snprintf(message_subject, sizeof(message_subject),
             "[Arrhythmia Detected] ECG Monitor Report %s (%03i:%i)",
             helperGetISODateTime(date_buffer),
             rtecg_config.device_id, rtecg_config.patient_id);

    /* Build body */
    snprintf(message_body, sizeof(message_body),
             "ECG Monitor Report: An arrhythmia was detected in %s (%i) by %s "
             "at approximately %s.\n\n"
             "Attached is the most recent log of raw data and a plot of last "
             "30 seconds of recorded ECG activity.",
             rtecg_config.patient_name,
             rtecg_config.patient_id,
             rtecg_config.device_name,
             helperGetISODateTime(date_buffer));

    /* Send message with attachments to both physician and patient */
    ret = sendMailAttachments((char*)rtecg_config.physician_email,
                              message_subject, message_body,
                              2, pngfile, logfile);
    if (ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "commTransmit(): Error at sendMailAttachments().");
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    ret = sendMailAttachments((char*)rtecg_config.patient_email,
                              message_subject, message_body,
                              2, pngfile, logfile);
    if (ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "commTransmit(): Error at sendMailAttachments().");
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    return 0;
}



/* commGetNewestLogfile(): Get the filename (not path) of the newest logfile 
   in the project's logfile directory, probably /var/rtecg or something like 
   that. This is done by getting a directory listing sorted alphabetically,
   and filtering out all files but those matching *.log.
   Input: None
   Output: filename
   Return: 0 on success, 1 on failure
*/
int commGetNewestLogfile(char* filename)
{
    int i;
    int ret;
    struct dirent **namelist;
    char msg_buffer[200];
    char filename_buffer[PROJECT_PATH_MAX];

    ret = scandir(rtecg_config.logfile_dir, &namelist, 0, alphasort);
    if (ret < 0) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "commGetFilename: Error at scandir().");
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    for (i=(ret-1); i>=0; i--) {
        if ((strncmp(namelist[i]->d_name, ".", strlen(".")) !=0) &&
            (strncmp(namelist[i]->d_name, "..", strlen("..")) != 0) &&
            (strlen(namelist[i]->d_name) != 0) &&
            (strstr(namelist[i]->d_name, ".log") != NULL)) {
            snprintf(filename_buffer, sizeof(filename_buffer),
                     "%s%s",
                     rtecg_config.logfile_dir, namelist[i]->d_name);
            strncpy(filename, filename_buffer, strlen(filename_buffer));
            free(namelist);
            break;
        }
    }

    return 0;
}
    
