/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   gnuplot.c - Software interface for calling gnuplot(1) to generate plots of 
   ECG data
   Calvin Walden
   2017-03-20
*/

#define _POSIX_C_SOURCE 2
#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "../../common/config.h"
#include "helper.h"
#include "gnuplot.h"


/* gnuplotPlotFile(): Given a logfile from which to pull data, generate a plot 
   its newest data, at the end of the file.
   Input: logfile (absolute path, file must exist) and pngfile (absolute path,
   file will be created)
   Output: Stored in file specified in pngfile
   Return: 0 on success, 1 on failure
*/
int gnuplotPlotFile(char* logfile, char* pngfile)
{
    FILE* fd;
    char msg_buffer[200];
    char command_buffer[100];
    char time_buffer[30];
    char* commands[] = { /* 0 */ "set term png\n", 
                         /* 1 */ "set term png size %i, %i\n",
                         /* 2 */ "set output \"%s\"\n",
                         /* 3 */ "set title \"ECG Data: Past 30 seconds of log\n",
                         /* 4 */ "set label \"Date: %s; Patient: %s (%i); "
                         "Device: (%03i)\" at screen 0.42, 0.93 font \"Arial,8\"\n",
                         /* 5 */ "plot [%i:%i] \"< tail -%i %s\" u 1 w lines "
                         "title \"Filtered signal\"\n" };

    snprintf(msg_buffer, sizeof(msg_buffer),
             "gnuplotPlotFile: Plotting contents of %s to %s.",
             logfile, pngfile);
    syslog(LOG_NOTICE, msg_buffer);

    /* Open the pipe for GNUPlot */
    snprintf(command_buffer, sizeof(command_buffer),
             "%s - persistent",
             rtecg_config.gnuplot_bin);
    fd = popen(command_buffer, "w");
    if (fd == NULL) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "gnuplotPlotFile: Error at popen(): %s", strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    /* Play fill-in-the-blank for the commands */
    fprintf(fd, commands[0]);
    fprintf(fd, commands[1], 1600, 640);
    fprintf(fd, commands[2], pngfile);
    fprintf(fd, commands[3]);
    fprintf(fd, commands[4], helperGetISODateTime(time_buffer),
            rtecg_config.patient_name,
            rtecg_config.patient_id, rtecg_config.device_id);
    fprintf(fd, commands[5], 0, 30*SENSOR_BUFFER,
             30*SENSOR_BUFFER, logfile);
    
    fflush(fd);
    pclose(fd);

    /* Check to see that the file was generated */
    if (access(pngfile, F_OK) == -1) {
        syslog(LOG_ERR, "gnuPlotFile: PNG file was not generated.");
        return 1;
    } 

    return 0;
}

