/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   helper.c - Helper functions
   ECG data
   Calvin Walden
   2017-03-20
*/

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <syslog.h>
#include <sys/statvfs.h>
#include <stdint.h>
#include <math.h>
#include <time.h>

#include "helper.h"



const unsigned int MiB = (1024*1024);

/* helperGetFreeSpace(): Get free space of the volume mounted at the path for
   directory using statvfs. Return values are in MiB.
   Input: directory: pathname to directory to run stat on
   Ouput: free: free space (free blocks X block size), in MiB
          total: total space (total blocks X block size), in MiB
   Return: 0 on success, 1 on failure.
*/
int helperGetFreeSpace(char* directory, int* free,
                       int* total, int* percent_used)
{
    int ret;
    char msg_buffer[500];
    struct statvfs stats = {0};
    int fs_total;
    int fs_available;
    int fs_used;
    int fs_percent_used;

    ret = statvfs(directory, &stats);
    if (ret == -1) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "helperGetFreeSpace: Error at statvfs: %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    fs_total = ceil((double)(stats.f_blocks * stats.f_frsize) / MiB);
    fs_available = ceil((double)(stats.f_bfree * stats.f_frsize) / MiB);
    fs_used = fs_total - fs_available;
    fs_percent_used = ceil(((double)fs_used / (double)fs_total) * (double)100);

#ifdef DEBUG_HELPER
    snprintf(msg_buffer, sizeof(msg_buffer),
             "Free space for %s: "
             "%lu blocks free of %lu blocks available with %lu block size, or "
             "%i MiB total, %i MiB available, %i MiB used, %i percent used",
             directory,
             stats.f_bfree, stats.f_blocks, stats.f_bsize,
             fs_total, fs_available, fs_used, fs_percent_used);
    syslog(LOG_NOTICE, msg_buffer);
#endif

    *free = fs_available;
    *total = fs_total;
    *percent_used = fs_percent_used;

    return 0;
}



/* helperGetISODateTime Get the current date and time string in an 
   ISO-8601-ish format 
   Input: buffer, which should be 20 characters long ("YYYY-MM-DD HH:MM:SS\0")
   Output: buffer (also)
   Return: *buffer, for convenience
*/
char* helperGetISODateTime(char* buffer)
{
    time_t rawtime;
    struct tm timeinfo;

    rawtime = time(NULL);
    timeinfo = *localtime(&rawtime);
    snprintf(buffer, 22,
             "%04i-%02i-%02i %02i:%02i:%02i",
             timeinfo.tm_year + 1900,
             timeinfo.tm_mon + 1,
             timeinfo.tm_mday,
             timeinfo.tm_hour,
             timeinfo.tm_min,
             timeinfo.tm_sec);

    return buffer;
}



/* helperGetISODate: Get the current date string in an ISO-8601-ish format
   ISO-8601-ish format 
   Input: buffer, which should be 11 characters long ("YYYY-MM-DD\0")
   Output: buffer (also)
   Return: *buffer, for convenience
*/
char* helperGetISODate(char* buffer)
{
    time_t rawtime;
    struct tm timeinfo;

    rawtime = time(NULL);
    timeinfo = *localtime(&rawtime);
    snprintf(buffer, 11,
             "%04i-%02i-%02i",
             timeinfo.tm_year + 1900,
             timeinfo.tm_mon + 1,
             timeinfo.tm_mday);

    return buffer;
}

