/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   mail.c - Functionality for sending email messages
   Calvin Walden
   2017-04-16
*/

#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdarg.h>

#include "../../common/config.h"
#include "../../common/rtecg.h"
#include "mail.h"
#include "net.h"
#include "helper.h"



/* sendMail: Send an email to a single recipient, with subject and body by 
   calling mail(1) using system(3).
   Input: recipient: Recipient's email address
          subject: Subject of email
          body: Body text of email
   Output: None
   Return: 0 on success, 1 on failure

   This function may block the running thead for a long period of 
   time ( > 1 second) while mail(1) sends the message.
*/
int sendMail(char* recipient, char* subject, char* body)
{
    int ret;
    char msg_buffer[200];
    char command_buffer[1024];
    
    /* Debug message */
    snprintf(msg_buffer, sizeof(msg_buffer),
             "sendMail: Sending email \"%s\" to %s",
             subject, recipient);
    syslog(LOG_NOTICE, msg_buffer);

    /* Build command buffer */
    snprintf(command_buffer, sizeof(command_buffer),
             "echo \"%s\" | mail -s \"%s\" %s",
             body, subject, recipient);

#ifdef DEBUG_MAIL
    syslog(LOG_NOTICE, command_buffer);
#endif

    /* Send the message */
    ret = system(command_buffer);
    if (ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "sendMail: Error at system(): %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }
    
    return 0;
}



/* sendMailStartup(): Send a system startup notification message using
   sendMail(). This message contains information about  system configuration
   (assigned name, ID, etc) and system status (interface IP addresses and 
   filesystem free space).
   Input: None
   Output: None
   Return value: 0 on success, 1 on failure
 */
int sendMailStartup(void) {
    int ret;
    int error = 0;
    char body_buffer[1000];
    char subject_buffer[200];
    char ip_wlan[NET_IP_LENGTH];
    char ip_eth[NET_IP_LENGTH];

    int sdcard_free;
    int sdcard_total;
    int sdcard_percent_used;

    char time_string[30];
    
    /* Get IP addresses */
    ret = netGetIP("eth0", ip_eth, sizeof(ip_eth));
    if (ret) {
        syslog(LOG_ERR, "sendMailStartup: (Non-fatal) "
               "Error at netGetIP (eth0)");
        error++;
    }
    
    ret = netGetIP("wlan0", ip_wlan, sizeof(ip_wlan));
    if (ret) {
        syslog(LOG_ERR, "sendMailStartup: (Non-fatal) "
               "Error at netGetIP (wlan0)");
        error++;
    }

    if (error > 1) {
        syslog(LOG_ERR, "sendMailStartup: Error: Could not get IP address for "
               "either network interface; assuming that system has no "
               "connection and aborting.");
        return 1;
    }

    /* Get current date/time */
    helperGetISODateTime(time_string);       

    /* Get SD card free space */
    ret = helperGetFreeSpace((char*)rtecg_config.rtecg_dir, &sdcard_free,
                             &sdcard_total, &sdcard_percent_used);

    /* Form the message subject */
    snprintf(subject_buffer, sizeof(subject_buffer),
             "ECG Monitor Startup Notification (%03i:%i)",
             rtecg_config.device_id, rtecg_config.patient_id);

    /* Form the message body */
    snprintf(body_buffer, sizeof(body_buffer),
             "%s, monitoring %s (%i), started up at %s and has the following "
             "IP addresses.\n"
             "\tWired Ethernet (eth0):\t%s\n"
             "\tWireless (wlan0):\t%s\n"
             "\n"
             "Local storage status: %i MiB of %i MiB available for %s "
             "(%i%% used).\n",
             rtecg_config.device_name,
             rtecg_config.patient_name,
             rtecg_config.patient_id,
             time_string,
             ip_eth, ip_wlan,
             sdcard_free, sdcard_total, rtecg_config.rtecg_dir,
             sdcard_percent_used);

    /* Send the message */
    ret = sendMail((char*)rtecg_config.physician_email,
                   subject_buffer,
                   body_buffer);

    return ret;
}



/* sendMailAttachments: Send an email to a single recipient, with subject, body, and
   an indeterminate number of attachments by calling mail(1) using system(3).
   Input: recipient: Recipient's email address
          subject: Subject of email
          body: Body text of email
          num_attachments: Number of attachments to follow
          ...: List of char*s of absolute paths of files to include.
   Output: None
   Return: 0 on success, 1 on failure

   Just like sendMail(), this function may block the running thread for a long
   period of time ( > 1 second) while mail(1) sends the message.

   As it is currently implemented, this function does no sanity checking on 
   the attachment paths, it just inserts them into the command string passed 
   to system(3).
*/
int sendMailAttachments(char* recipient, char* subject, char* body,
                        int num_attachments, ...)
{
    int i;
    int ret;
    va_list attachments;
    char attachment_buffer[4096];
    char msg_buffer[200];
    char command_buffer[4096];
    int offset;

    /* Build attachment argument buffer */
    va_start(attachments, num_attachments);
    offset = 0;
    for (i=0; i<num_attachments; i++) {
        offset += snprintf((attachment_buffer+offset),
                           sizeof(attachment_buffer)-offset,
                           " -A %s",
                           va_arg(attachments, char*));
    }
    va_end(attachments);
    
    /* Debug message */
    snprintf(msg_buffer, sizeof(msg_buffer),
             "sendMailAttachments: Sending email \"%s\" to %s",
             subject, recipient);
    syslog(LOG_NOTICE, msg_buffer);

    /* Build command buffer */
    snprintf(command_buffer, sizeof(command_buffer),
             "echo \"%s\" | mail %s -s \"%s\" %s",
             body, attachment_buffer, subject, recipient);

#ifdef DEBUG_MAIL
    syslog(LOG_NOTICE, command_buffer);
#endif

    /* Send the message */
    ret = system(command_buffer);
    if (ret) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "sendMailAttachments: Error at system(): %s",
                 strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    return 0;
}

