/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   main.c - Main source file
   Calvin Walden
   2017-03-20
*/

#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "../../common/config.h"
#include "../../common/rtecg.h"
#include "comm.h"
#include "mail.h"
#include "net.h"
#include "signalhandler.h"



int main(int argc, char *argv[]) {
    int ret;
    pid_t pid;
    pid_t sid;
    char msg_buffer[200];
 
    /* Fork off of the parent process */
    pid = fork();

    if (pid < 0) {
        exit(EXIT_FAILURE);
        /* Unable to fork() for some reason. */
    } else if (pid > 0) {
        exit(EXIT_SUCCESS);
    }

    /* At this point, this is the child process executing. */
    umask(0);

    /* Open up logfile */
    openlog(argv[0], LOG_NOWAIT | LOG_PID, LOG_USER);

    /* First syslog message */
    syslog(LOG_NOTICE, "Started Real-Time ECG Communication Daemon.\n");

    /* Get our own session ID */
    sid = setsid();
    if (sid < 0) {
        syslog(LOG_ERR, "main: Could not create process group.\n");
        exit(EXIT_FAILURE);
    }

    /* Set up signal handlers */
    ret = signalHandlerSetup();
    if (ret) {
        syslog(LOG_ERR, "main: Error at signalHandlerSetup().");
        exit(EXIT_FAILURE);
    }

    /* Close stdio file descriptors, since I don't think we really 
       need them. */
    close(STDIN_FILENO);
    close(STDOUT_FILENO);
    close(STDERR_FILENO);
    
    /* Load configuration files */
    ret = configLoad();
    if (ret) {
        syslog(LOG_ERR, "main: Error loading rtecg.cfg. Exiting.");
        exit(EXIT_FAILURE);
    }
    
    /* Check for the mail binary */
    if (access(rtecg_config.mail_bin, F_OK) == -1) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "main: Could not find binary for mail: %s. Exiting.\n",
                 rtecg_config.mail_bin);
        syslog(LOG_ERR, msg_buffer);
        exit(EXIT_FAILURE);
    }

    /* Check for the gnuplot binary */
    if (access(rtecg_config.gnuplot_bin, F_OK) == -1) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "main: Could not find binary for gnuplot: %s. Exiting.\n",
                 rtecg_config.gnuplot_bin);
        syslog(LOG_ERR, msg_buffer);
        exit(EXIT_FAILURE);
    }

    /* Wait for network interfaces to come up, if they're not already */
    ret = netWaitForInterface("wlan0");
    if (ret) {
        syslog(LOG_ERR, "main: Could not establish network connectivity. "
               "Exiting.\n");
        exit(EXIT_FAILURE);
    }

    /* If all went well, send an email notification for the service startup */
    ret = sendMailStartup();
    if (ret) {
        syslog(LOG_ERR, "main: Could not send startup notification email.");
    }

    /* Enter main loop where we'll wait for signals */
    commLoop();

    /* ...And we're done here. */
    closelog();

    return 0;
}
