/* Real-Time Electrocardiogram Monitoring
   Communication Daemon
   net.c - Functions for getting network interface status
   Calvin Walden
   2017-03-20
*/

#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <linux/if.h>

#include "net.h"



/* netGetIP: Given the name of a network interface, get its IPv4 address.
   Input: interface (probably "eth0" or "wlan0")
          ip_len - length of string char* interface
   Output: ip
   Return: 0 on success, 1 on failure

   Based loosely on code found here:
   http://stackoverflow.com/questions/2283494/
 */
int netGetIP(char* interface, char* ip, int ip_len)
{
    int fd;
    int ret;
    char msg_buffer[200];
    struct ifreq ifr;

    memset(&ifr, 0, sizeof(ifr));

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd == -1) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "netGetIP (%s): Error at socket(): "
                 "%s", interface, strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    ifr.ifr_addr.sa_family = AF_INET;  /* We want the IPv4 address of 
                                          the interface. */

    strncpy(ifr.ifr_name, interface, IFNAMSIZ);

    ret = ioctl(fd, SIOCGIFADDR, &ifr);
    if (ret == -1) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "netGetIP (%s): Error at ioctl(): "
                 "%s", interface, strerror(errno));
        syslog(LOG_ERR, msg_buffer);
        return 1;
    }

    close(fd);

    snprintf(ip, ip_len, "%s",
             inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));

#ifdef DEBUG_NET
    syslog(LOG_NOTICE, ip);
#endif

    return 0;
}



/* netWaitForInterface: Using the same mechanism as netGetIP, wait for an 
   interface to come online, a condition defined here as being assigned
   an IP address by DHCP. This function will try up to NET_MAX_ATTEMPTS to
   read the assigned IP address of the interface, waiting NET_SLEEP_TIME 
   seconds before attempts. During this time, the function blocks the thread
   it is running in.
   Input: interface - name of network interface
   Output: None
   Return: 0 on success (interface had/acquired IP address), 1 on failure
*/
int netWaitForInterface(char* interface)
{
    int fd;
    int ret;
    char msg_buffer[200];
    struct ifreq ifr;
    int attempt_counter;

#ifdef DEBUG_NET
    snprintf(msg_buffer, sizeof(msg_buffer),
             "netWaitForInterface: %s", interface);
    syslog(LOG_NOTICE, msg_buffer);
#endif

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd == -1) {
        snprintf(msg_buffer, sizeof(msg_buffer),
                 "netWaitForInterface (%s): Error at socket(): %s",
                 interface, strerror(errno));
        syslog(LOG_ERR, msg_buffer);
    }

    attempt_counter = 0;

    while (1) {
        memset(&ifr, 0, sizeof(ifr));

        strncpy(ifr.ifr_name, interface, IFNAMSIZ);
        ifr.ifr_addr.sa_family = AF_INET;  /* We want the IPv4 address of 
                                          the interface. */

        /* We'll consider the interface to be "up" when it has an 
           IP address. */
        ret = ioctl(fd, SIOCGIFADDR, &ifr);

        if ((ret == -1) && (errno == EADDRNOTAVAIL)) {
            /* "Could not get requested address", indicates that the interface 
               does not yet have an IP address. */

            snprintf(msg_buffer, sizeof(msg_buffer),
                     "netWaitForInterface: Waiting for %s to come online...",
                     interface);
            syslog(LOG_NOTICE, msg_buffer);
        
            sleep(NET_SLEEP_TIME);
            attempt_counter++;
        } else if (ret == 0) {
            /* Everything's all good; the interface has an address. */
            break;
        } else {
            /* Any other error: ((ret == -1) && (errno != EADDRNOTAVAIL)) */
            snprintf(msg_buffer, sizeof(msg_buffer),
                     "netGetIP (%s): Error at ioctl(): "
                     "%s", interface, strerror(errno));
            syslog(LOG_ERR, msg_buffer);
            return 1;
        }

        if (attempt_counter == NET_MAX_ATTEMPTS) {
            snprintf(msg_buffer, sizeof(msg_buffer),
                     "netWaitForInterface: Exceeded maximum attempts (%i) "
                     " waiting for %s to come online. Giving up.",
                     NET_MAX_ATTEMPTS, interface);
            syslog(LOG_NOTICE, msg_buffer);

            close(fd);
            
            return 1;
        }
    }

    close(fd);

    return 0;
}

